diff --git a/lib/vfs/mc-vfs/fish.c b/lib/vfs/mc-vfs/fish.c
index 16c6ffc..e183308 100644
--- a/lib/vfs/mc-vfs/fish.c
+++ b/lib/vfs/mc-vfs/fish.c
@@ -907,13 +907,18 @@ fish_file_store (struct vfs_class *me, struct vfs_s_fh *fh, char *name, char *lo
 
     total = 0;
 
+    tty_enable_interrupt_key ();
     while (1)
     {
         int t;
         while ((n = read (h, buffer, sizeof (buffer))) < 0)
         {
-            if ((errno == EINTR) && tty_got_interrupt ())
+            if (tty_got_interrupt ())
+                goto error_return;
+
+            if (errno == EINTR)
                 continue;
+
             print_vfs_message (_("fish: Local read failed, sending zeros"));
             close (h);
             h = open ("/dev/zero", O_RDONLY);
@@ -922,6 +927,9 @@ fish_file_store (struct vfs_class *me, struct vfs_s_fh *fh, char *name, char *lo
         if (n == 0)
             break;
 
+        if (tty_got_interrupt ())
+            goto error_return;
+
         t = write (SUP.sockw, buffer, n);
         if (t != n)
         {
@@ -931,18 +939,19 @@ fish_file_store (struct vfs_class *me, struct vfs_s_fh *fh, char *name, char *lo
                 me->verrno = EIO;
             goto error_return;
         }
-        tty_disable_interrupt_key ();
         total += n;
         print_vfs_message ("%s: %d/%" PRIuMAX,
                            was_error ? _("fish: storing zeros") : _("fish: storing file"),
                            total, (uintmax_t) s.st_size);
     }
+    tty_disable_interrupt_key ();
     close (h);
     g_free (quoted_name);
     if ((fish_get_reply (me, SUP.sockr, NULL, 0) != COMPLETE) || was_error)
         ERRNOR (E_REMOTE, -1);
     return 0;
   error_return:
+    tty_disable_interrupt_key ();
     close (h);
     fish_get_reply (me, SUP.sockr, NULL, 0);
     g_free (quoted_name);
